/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/*                                                                 */
/* file: aGarciaDefs.tea                                           */
/*                                                                 */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/*                                                                 */
/* description: Defines for Garcia.                                */
/*                                                                 */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/*                                                                 */
/*        docs: Documentation remains TBD                          */
/*                                                                 */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/*                                                                 */
/* Copyright 1994-2008. Acroname Inc.                              */
/*                                                                 */
/* This software is the property of Acroname Inc.  Any             */
/* distribution, sale, transmission, or re-use of this code is     */
/* strictly forbidden except with permission from Acroname Inc.    */
/*                                                                 */
/* To the full extent allowed by law, Acroname Inc. also excludes  */
/* for itself and its suppliers any liability, wheither based in   */
/* contract or tort (including negligence), for direct,            */
/* incidental, consequential, indirect, special, or punitive       */
/* damages of any kind, or for loss of revenue or profits, loss of */
/* business, loss of information or data, or other financial loss  */
/* arising out of or in connection with this software, even if     */
/* Acroname Inc. has been advised of the possibility of such       */
/* damages.                                                        */
/*                                                                 */
/* Acroname Inc.                                                   */
/* www.acroname.com                                                */
/* 720-564-0373                                                    */
/*                                                                 */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */



#ifndef _aGarciaDefs_T_
#define _aGarciaDefs_T_

/* our latest dump */
#define aGARCIA_DUMP_NUM		23
#define aGARCIA_DUMP_REV		0

/* EEPROM lock check address */
#define	aGARCIA_LOCK_CHK_ADDR		0x03FF

/* unit constants */
#define aGARCIA_DISTANCE_METERS		0
#define aGARCIA_DISTANCE_FEET		1
#define aGARCIA_DISTANCE_INCHES		2
#define aGARCIA_ANGLE_RADIANS		0
#define aGARCIA_ANGLE_DEGREES		1
#define	aGARCIA_PID_PERIOD		80	/* 50,80 */
#define aGARCIA_MAX_VELOCITY		102	/* 64,102 */


/* drive train constants */
/*
* Gearbox produces 3648 ticks per outer wheel revolution
* (3648000)/(diam*pi) = ticks/m
* (base/diam)*(3648) = ticks/(360 deg)
* 3.28084 feet per meter
* wheel base = 180mm
* wheel diam = 101mm
*/
#define	aGARCIA_TICKS_PER_METER		11497
#define	aGARCIA_TICKS_PER_FOOT		3504
#define	aGARCIA_TICKS_PER_RADIAN	1035
#define	aGARCIA_TICKS_PER_DEGREE	18
#define	aGARCIA_TICKS_PER_360DEGREES	6501

/* error values */
#define aGARCIA_ERRFLAG_NORMAL		0x0000
#define aGARCIA_ERRFLAG_STALL		0x0001
#define aGARCIA_ERRFLAG_FRONTR_LEFT	0x0002
#define aGARCIA_ERRFLAG_FRONTR_RIGHT	0x0003
#define aGARCIA_ERRFLAG_REARR_LEFT	0x0004
#define aGARCIA_ERRFLAG_REARR_RIGHT	0x0005
#define aGARCIA_ERRFLAG_SIDER_LEFT	0x0008
#define aGARCIA_ERRFLAG_SIDER_RIGHT	0x0009
#define aGARCIA_ERRFLAG_FALL_LEFT	0x0010
#define aGARCIA_ERRFLAG_FALL_RIGHT	0x0011
#define aGARCIA_ERRFLAG_ABORT		0x0012
#define aGARCIA_ERRFLAG_NOTEXECUTED	0x0013
#define aGARCIA_ERRFLAG_WONTEXECUTE	0x0014
#define aGARCIA_ERRFLAG_BATT		0x0020
#define aGARCIA_ERRFLAG_IRRX		0x0040

/* rear sensor control flags */
#define aGARCIA_REARCTRL_DOCK		0x01
#define aGARCIA_REARCTRL_LEFT		0x02
#define aGARCIA_REARCTRL_RIGHT		0x04
#define aGARCIA_REARCTRL_INV		0x08
#define aGARCIA_REARCTRL_MASK		0x0F

/* motion termination flags */
#define aGARCIA_ENDCTRL_FRONT		0x02
#define aGARCIA_ENDCTRL_REAR		0x04
#define aGARCIA_ENDCTRL_SIDE		0x08
#define aGARCIA_ENDCTRL_IRRX		0x10

/* execution control flags */
#define aGARCIA_EXECTRL_USER		0x01
#define aGARCIA_EXECTRL_IRCHK		0x02
#define aGARCIA_EXECTRL_EDGECHK		0x04
#define aGARCIA_EXECTRL_STALLCHK	0x08

/* wallhug modes */
#define aGARCIA_HUGMODE_DEFAULT		0
#define aGARCIA_HUGMODE_SIDELOSS	1
#define aGARCIA_HUGMODE_DOCKFIND	2

/* align modes */
#define aGARCIA_ALIGNMODE_DEFAULT	0
#define aGARCIA_ALIGNMODE_ROTATE	1
#define aGARCIA_ALIGNMODE_STRAIGHT	2

/* default data */
#define aGARCIA_DOCK_THR		350



/* MOTO 1.0 BOARD */

/* Moto IO Constants */

#define	aGARCIA_MOTO_ADDR		4
#define aGARCIA_MOTO_MOTOR_RIGHT	0
#define aGARCIA_MOTO_MOTOR_LEFT		1
#define aGARCIA_MOTO_ARANGE_FRONT_LEFT	0
#define aGARCIA_MOTO_ARANGE_FRONT_RIGHT	1
#define aGARCIA_MOTO_ARANGE_SIDE_LEFT	2
#define aGARCIA_MOTO_ARANGE_SIDE_RIGHT	3
#define aGARCIA_MOTO_DLED_USER		0
#define aGARCIA_MOTO_DEDGE_RIGHT	5
#define aGARCIA_MOTO_DEDGE_LEFT		10

/* Moto Shared Data */

#define	aGARCIA_MOTO_PADS_STATUS	0
#define	aGARCIA_MOTO_PADB_MOVEDONE	2
#define	aGARCIA_MOTO_PADB_ENDFLAGS	3
#define aGARCIA_MOTO_PADB_GPSTAT	4
#define aGARCIA_MOTO_PADB_EXEFLAGS	5
#define aGARCIA_MOTO_PADB_MONENABLE	6
#define	aGARCIA_MOTO_PADB_MIRRORIO	7
#define	aGARCIA_MOTO_PADS_STALLTHR	8
#define	aGARCIA_MOTO_PADS_FRONTTHR	10
#define	aGARCIA_MOTO_PADS_SIDETHR	12
#define	aGARCIA_MOTO_PADS_DEFVEL	14
#define	aGARCIA_MOTO_PADS_DEFACCTM	16
#define	aGARCIA_MOTO_PADS_NULL_LVEL	24
#define	aGARCIA_MOTO_PADS_NULL_RVEL	26
#define aGARCIA_MOTO_CTR_STALLMASK	5
#define aGARCIA_MOTO_CTR_IRRX		6
#define	aGARCIA_MOTO_BIT_LEDUSER	0


/* Moto file, process, and reflex assignments */

#define aGARCIA_MOTO_FILE_SWPIVOT	0
#define aGARCIA_MOTO_FILE_MOVE		1
#define aGARCIA_MOTO_FILE_PIVOT         2
#define aGARCIA_MOTO_FILE_WALLHUG	3
#define aGARCIA_MOTO_FILE_ALIGN		4
#define aGARCIA_MOTO_FILE_NULL		5
#define aGARCIA_MOTO_FILE_DOCK		7
#define aGARCIA_MOTO_FILE_CONFIG	8
#define aGARCIA_MOTO_FILE_MONITOR	9
#define aGARCIA_MOTO_FILE_DEMO		10
#define aGARCIA_MOTO_FILE_USER		11
#define aGARCIA_MOTO_PID_BEHAVIOR	0
#define aGARCIA_MOTO_PID_MONITOR	1
#define aGARCIA_MOTO_PID_USER		2
#define aGARCIA_MOTO_RFLX_RAMP		125
#define aGARCIA_MOTO_RFLX_LED		124


/* GP 2.0 BOARD */

/* GP IO Constants */

#define	aGARCIA_GP_ADDR			2
#define aGARCIA_GP_ARANGE_REAR_LEFT	0
#define aGARCIA_GP_ARANGE_REAR_RIGHT	1
#define aGARCIA_GP_ABATTERY		2
#define aGARCIA_GP_ACURRENT_RMOTOR	3
#define aGARCIA_GP_ACURRENT_LMOTOR	4
#define aGARCIA_GP_DENABLE_SIDERNG	0
#define aGARCIA_GP_DENABLE_FRONTRNG	1
#define aGARCIA_GP_DENABLE_REARRNG	2
#define aGARCIA_GP_DENABLE_DOWNRNG	3
#define aGARCIA_GP_DIRCOMM_RX		4
#define aGARCIA_GP_DIRCOMM_TX		5
#define aGARCIA_GP_DBUTTON		6
#define	aGARCIA_GP_SERVO_PAN		0
#define	aGARCIA_GP_SERVO_TILT		1

/* GP Shared Data */

#define aGARCIA_GP_PADB_REARCTRL	0
#define	aGARCIA_GP_PADB_MIRRORIO	1
#define	aGARCIA_GP_PADB_MIRRORMASK	2
#define	aGARCIA_GP_PADB_MIRRORBUTTON	3
#define	aGARCIA_GP_PADB_STASHID		4
#define aGARCIA_GP_CTR_REARTHR		7
#define	aGARCIA_GP_BIT_ENABLESIDE	0
#define	aGARCIA_GP_BIT_ENABLEFRONT	1
#define	aGARCIA_GP_BIT_ENABLEREAR	2
#define	aGARCIA_GP_BIT_ENABLEDOWN	3


/* GP file, process, and reflex assignments */

#define aGARCIA_GP_FILE_MONITOR		0
#define aGARCIA_GP_PID_MONITOR		0


/* misc codes used by reflexes */

#define	aGARCIA_CODE_SIDE_LEFT		2
#define	aGARCIA_CODE_SIDE_RIGHT		3
#define	aGARCIA_CODE_FRONT_LEFT		0
#define	aGARCIA_CODE_FRONT_RIGHT	1
#define	aGARCIA_CODE_REAR_LEFT		4
#define	aGARCIA_CODE_REAR_RIGHT		5
#define	aGARCIA_CODE_DOWN_RIGHT		10
#define	aGARCIA_CODE_DOWN_LEFT		15


/* constants specific to the Robot API */

#define	aGARCIA_APIFAC_METERS		11497
#define	aGARCIA_APIFAC_MPS		92
#define	aGARCIA_APIFAC_BATT		3248
#define	aGARCIA_APIFAC_GP2D12		400
#define	aGARCIA_APIFAC_RADIANS		1035
#define	aGARCIA_APIFAC_RADIANS2		2070

#endif /* _aGarciaDefs_T_ */
